import argparse
from pathlib import Path
from app.preprocessing import preprocess

DATASET_DIR_ARG = '--dataset-dir'
DATASET_DIR_ARG_SHORT = '-d'
OUT_DIR_ARG = '--out'
OUT_DIR_ARG_SHORT = '-o'
TEST_DATA_FILES_ARG = '--test-data-files'
EXCLUDED_FILES_ARG = '--excluded-files'
PARALLELIZATION_ARG = '--parallelization'

def main():
    argument_parser = argparse.ArgumentParser('maneuver-detection', description='Maneuver Detection for the Loft Dynamics Helicopter Simulator')
    subparsers = argument_parser.add_subparsers(title='action')

    preprocess_parser = subparsers.add_parser('preprocess')
    preprocess_parser.add_argument(DATASET_DIR_ARG, DATASET_DIR_ARG_SHORT, action='store', type=Path, required=True)
    preprocess_parser.add_argument(OUT_DIR_ARG, OUT_DIR_ARG_SHORT, action='store', type=Path, required=True)
    preprocess_parser.add_argument(TEST_DATA_FILES_ARG, action='store', type=str, required=False, nargs='*', default=None, help="Only the filenames without the extension")
    preprocess_parser.add_argument(EXCLUDED_FILES_ARG, action='store', type=str, required=False, nargs='*', default=None)
    preprocess_parser.add_argument(PARALLELIZATION_ARG, action=argparse.BooleanOptionalAction, type=bool, required=False, default=True)
    preprocess_parser.set_defaults(func=lambda dataset_dir, out, test_data_files, excluded_files, parallelization, func: preprocess(dataset_dir, out, test_data_files, excluded_files, parallelization))

    parsed_args = argument_parser.parse_args()
    args = vars(parsed_args)
    parsed_args.func(**args)

if __name__ == "__main__":
    main()
