import numpy as np
from sklearn.discriminant_analysis import QuadraticDiscriminantAnalysis
from sklearn.feature_selection import SequentialFeatureSelector
from sklearn.metrics import accuracy_score, make_scorer
import pandas as pd
from sklearn.model_selection import cross_val_score

# EVALUATION_FILES = [
#     Path('out/artifacts/blocks/10/39b2c145-c49f-470b-8280-d253fa98153f.parquet'),
#     Path('out/artifacts/blocks/10/92b2d28b-21e4-498c-b6dd-c27a47716a25.parquet'),
#     Path('out/artifacts/blocks/10/ef4852a4-fcfe-429b-b753-d11e2ad08cac.parquet'),
# ]

# Simulated dataset chunk loader (replace with actual logic)
# def load_file():
#     for file in EVALUATION_FILES:
#         df = pd.read_parquet(file)
#         X = df.drop(columns=['Maneuver'])
#         y = df['Maneuver']

#         yield X.to_numpy(), y.to_numpy()
#     # for _ in range(5):  # Simulate 5 chunks
#     #     X_chunk = np.random.rand(chunk_size, 10)  # 10 features
#     #     y_chunk = np.random.choice([0, 1], size=chunk_size)  # Binary classes
#     #     yield X_chunk, y_chunk

# # Custom scorer for SFS with incremental processing
# def incremental_scorer(estimator, X, y, selected_features):
#     """
#     Incrementally evaluate model performance for a subset of features.
#     """
#     total_predictions = []
#     total_true = []

#     for X_chunk, y_chunk in load_file():
#         # Select only the relevant features
#         X_chunk_selected = X_chunk[:, selected_features]

#         # Make predictions using the current model
#         estimator.fit(X_chunk_selected, y_chunk)
#         y_pred = estimator.predict(X_chunk_selected)

#         # Store predictions and true labels
#         total_predictions.extend(y_pred)
#         total_true.extend(y_chunk)

#     # Compute and return the metric (accuracy in this case)
#     return accuracy_score(total_true, total_predictions)

def evaluate_features():
    print('Evaluating features')
    # df = pd.read_parquet('out/artifacts/blocks/20/39b2c145-c49f-470b-8280-d253fa98153f.parquet')
    # df = pd.read_parquet('out/artifacts/blocks/20/92b2d28b-21e4-498c-b6dd-c27a47716a25.parquet')
    df = pd.read_parquet([
        'out/artifacts/blocks/20/39b2c145-c49f-470b-8280-d253fa98153f.parquet',
        'out/artifacts/blocks/20/92b2d28b-21e4-498c-b6dd-c27a47716a25.parquet',
    ])
    X_df = df.drop(columns=['Maneuver'])
    y_df = df['Maneuver']
    X = X_df.to_numpy()
    y = y_df.to_numpy()

    qda = QuadraticDiscriminantAnalysis()
    scorer = make_scorer(accuracy_score)

    # best_score = -np.inf
    # best_features = []
    # for n_features in range(1, max_features):
        # print(f'Running with {n_features} features')
    sfs = SequentialFeatureSelector(
        qda,
        direction='forward',
        scoring=scorer,
        cv=5,
        n_jobs=-1,
        tol=1e-3
    )

    # Simulate fitting the SFS on an "empty" dataset (data loading is done in chunks)
    # X_placeholder = np.zeros((1, 10))  # Placeholder for 10 features
    # y_placeholder = np.zeros(1)  # Placeholder for labels
    sfs.fit(X, y)

    # Get selected features
    selected_features = sfs.get_support()
    print(f'Selected Features (index): {selected_features}')

    column_names = [df.columns[idx] for idx in range(len(selected_features)) if selected_features[idx]]
    print(f'Selected Features (name): {column_names}')

    X_selected = X[:, selected_features]
    score = cross_val_score(qda, X_selected, y, cv=5, scoring='accuracy').mean()
    print(f'Best score: {score}')

    return column_names, score


if __name__ == '__main__':
    evaluate_features()

    # features = []

    # df = pd.read_parquet('out/artifacts/blocks/10/92b2d28b-21e4-498c-b6dd-c27a47716a25.parquet')
    # X_df = df[features]
    # y_df = df['Maneuver']
    # qda = QuadraticDiscriminantAnalysis()

    # score = cross_val_score(qda, X_df.to_numpy(), y_df.to_numpy(), cv=5, scoring='accuracy').mean()
    # print(f'CV Score: {score}')
