from typing import List, Tuple
from pathlib import Path
import shutil
import os

import utils as preprocessing_utils
import common_filenames
from file_type import FileType

def _train_test_split(files: List[Path], test_data_files: List[str]) -> Tuple[List[Path], List[Path]]:
    train_files: List[Path] = []
    test_files: List[Path] = []

    for file in files:
        if file.stem in test_data_files:
            test_files.append(file)
        else:
            train_files.append(file)

    return train_files, test_files

def exclude_files_by_stem(files: List[Path], excluded_files: List[str]) -> List[Path]:
    return [file for file in files if file.stem not in excluded_files]

def _copy_files(files: List[Path], destination: Path) -> None:
    for file in files:
        destination_filename = destination / file.name
        shutil.copyfile(file, destination_filename)

def group_dataset(dataset_dir: Path, out_dir: Path, test_data_files: List[str] = None, excluded_files: List[str] = None) -> None:
    test_data_files = test_data_files if test_data_files is not None else common_filenames.TEST_DATA_FILES
    excluded_files = excluded_files if excluded_files is not None else common_filenames.EXCLUDED_FILES

    preprocessing_utils.recreate_dir(out_dir)

    training_dir = out_dir / 'training'
    test_dir = out_dir / 'test'

    parquet_subdir = 'parquet'
    json_subdir = 'json'
    training_parquet_dir = training_dir / parquet_subdir
    training_json_dir = training_dir / json_subdir
    test_parquet_dir = test_dir / parquet_subdir
    test_json_dir = test_dir / json_subdir
    
    os.makedirs(training_parquet_dir, exist_ok=True)
    os.makedirs(training_json_dir, exist_ok=True)
    os.makedirs(test_parquet_dir, exist_ok=True)
    os.makedirs(test_json_dir, exist_ok=True)

    parquet_files = preprocessing_utils.files_from_dataset(dataset_dir, FileType.Parquet)
    json_files = preprocessing_utils.files_from_dataset(dataset_dir, FileType.Json)

    parquet_files = exclude_files_by_stem(parquet_files, excluded_files)
    json_files = exclude_files_by_stem(json_files, excluded_files)

    train_parquet_files, test_parquet_files = _train_test_split(parquet_files, test_data_files)
    train_json_files, test_json_files = _train_test_split(json_files, test_data_files)

    _copy_files(train_parquet_files, training_parquet_dir)
    _copy_files(test_parquet_files, test_parquet_dir)
    _copy_files(train_json_files, training_json_dir)
    _copy_files(test_json_files, test_json_dir)
