from __future__ import annotations
from typing import Dict
from datetime import datetime
import pytz

class JsonManeuverData:
    _label: str
    _start_timestamp: datetime
    _end_timestamp: datetime

    TIMESTAMP_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"
    NO_MANEUVER_LABEL = "no maneuver"

    @property
    def label(self) -> str:
        return self._label
    
    @property
    def start_timestamp(self) -> datetime:
        return self._start_timestamp
    
    @property
    def end_timestamp(self) -> datetime:
        return self._end_timestamp

    def __init__(self, label: str, start_timestamp: datetime, end_timestamp: datetime):
        self._label = label
        self._start_timestamp = start_timestamp
        self._end_timestamp = end_timestamp

    @staticmethod
    def fromJson(json_data: Dict[str, str]) -> JsonManeuverData:
        label = json_data['comment'].lower()
        start_timestamp = datetime.strptime(json_data['startTimestamp'], JsonManeuverData.TIMESTAMP_FORMAT).replace(tzinfo=pytz.UTC)
        end_timestamp = datetime.strptime(json_data['endTimeStamp'], JsonManeuverData.TIMESTAMP_FORMAT).replace(tzinfo=pytz.UTC)

        return JsonManeuverData(label, start_timestamp, end_timestamp)
