from typing import List, Callable
import shutil
import os
from pathlib import Path
import pandas as pd

from .file_type import FileType
from . import common_filenames

def recreate_dir(dir: Path) -> None:
    if dir.exists():
        shutil.rmtree(dir)

    os.makedirs(dir)

def files_from_dataset(dataset_dir: Path, file_type: FileType) -> List[Path]:
    return _files_from_dataset_where(dataset_dir, file_type)

def is_column_of_type(column: pd.Series, type: type) -> bool:
    return isinstance(column.values[0], type)

def _files_from_dataset_where(dataset_dir: Path, file_type: FileType, predicate: Callable[[Path], bool] = None) -> List[Path]:
    if predicate is None:
        predicate = lambda _: True
    return [path for path in dataset_dir.glob(f'*{file_type.file_extension}') if path.is_file() and predicate(path)]

def train_files_from_dataset(dataset_dir: Path, file_type: FileType = FileType.Parquet) -> List[Path]:
    return _files_from_dataset_where(dataset_dir, file_type, lambda p: p.stem not in common_filenames.TEST_DATA_FILES)

def test_files_from_dataset(dataset_dir: Path, file_type: FileType = FileType.Parquet) -> List[Path]:
    return _files_from_dataset_where(dataset_dir, file_type, lambda p: p.stem in common_filenames.TEST_DATA_FILES)
