import numpy as np
from sklearn.discriminant_analysis import QuadraticDiscriminantAnalysis
import pandas as pd
from pathlib import Path
from typing import List
from sklearn.metrics import accuracy_score
from sklearn.metrics import recall_score

FILES_PATH = Path('dataset/preprocessing/transformed_data/full_dataset_transformation/bool_as_int')

TEST_FILENAMES = [
    "0b3f3902-2c04-4625-8576-3bb963e3d709",
    "663f573a-74c5-4368-b60b-1fb433cd835d",
    "8c36586f-94e9-4ae9-8384-0f3342008677",
    "a376807a-82d3-4526-b19f-98d4b3f9078b",
    "d76bb0eb-bc08-4b35-8c1f-37369452083d",
    "f40f71de-5cc2-4719-8a5a-abcf950cbd71",
]

TRAIN_FILES = [path for path in FILES_PATH.glob(f'*.parquet') if path.is_file() and path.stem not in TEST_FILENAMES]
TEST_FILES = [path for path in FILES_PATH.glob(f'*.parquet') if path.is_file() and path.stem in TEST_FILENAMES]

# Simulated dataset chunk loader (replace with your data loader)
def load_chunk(files: List[Path]):
    """
    Generator function to simulate loading chunks of data.
    Replace this with your data loading logic.
    Example: yield X_chunk, y_chunk
    """
    # for _ in range(5):  # Simulate 5 chunks
    #     X_chunk = np.random.rand(chunk_size, 10)  # 10 features
    #     y_chunk = np.random.choice([0, 1], size=chunk_size)  # Binary classes
    #     yield X_chunk, y_chunk

    for file in files:
        print(f'Loading file {file.stem}')
        df = pd.read_parquet(file)
        X_df = df.drop(columns=['Maneuver'])
        y_df = df['Maneuver']
        yield X_df.to_numpy(), y_df.to_numpy()

# # Initialize variables
# class_sums = {}
# class_counts = {}
# class_covariances = {}

# total_samples = 0

# # First pass: Compute class means and priors
# for X_chunk, y_chunk in load_chunk(TRAIN_FILES):
#     unique_classes = np.unique(y_chunk)
#     for cls in unique_classes:
#         X_cls = X_chunk[y_chunk == cls]
#         if cls not in class_sums:
#             class_sums[cls] = np.zeros(X_cls.shape[1])
#             class_counts[cls] = 0
#         class_sums[cls] += np.sum(X_cls, axis=0)
#         class_counts[cls] += X_cls.shape[0]
#     total_samples += len(y_chunk)

# # Compute class means and priors
# class_means = {cls: class_sums[cls] / class_counts[cls] for cls in class_sums}
# priors = {cls: class_counts[cls] / total_samples for cls in class_counts}

# # Initialize covariance placeholders
# for cls in class_means:
#     class_covariances[cls] = np.zeros((len(class_means[cls]), len(class_means[cls])))

# # Second pass: Compute covariances
# for X_chunk, y_chunk in load_chunk(TRAIN_FILES):
#     unique_classes = np.unique(y_chunk)
#     for cls in unique_classes:
#         X_cls = X_chunk[y_chunk == cls]
#         mean_diff = X_cls - class_means[cls]
#         class_covariances[cls] += mean_diff.T @ mean_diff

# # Normalize covariance matrices
# for cls in class_covariances:
#     class_covariances[cls] /= (class_counts[cls] - 1)

# # Fit QDA using precomputed parameters
# qda = QuadraticDiscriminantAnalysis(store_covariance=True)
# qda.means_ = np.array([class_means[cls] for cls in sorted(class_means.keys())])
# qda.covariance_ = np.array([class_covariances[cls] for cls in sorted(class_covariances.keys())])
# qda.priors_ = np.array([priors[cls] for cls in sorted(priors.keys())])

qda = QuadraticDiscriminantAnalysis()
df = pd.read_parquet(TRAIN_FILES)
X_df = df.drop(columns=['Maneuver'])
y_df = df['Maneuver']

qda.fit(X_df.to_numpy(), y_df.to_numpy())

# Ready to use the model for predictions
print("QDA model fitted successfully with precomputed parameters!")

y_true = []
y_pred = []

n_rows = 0
for X, chunk_y_true in load_chunk(TEST_FILES):
    chunk_y_pred = qda.predict(X)
    y_true.extend(chunk_y_true)
    y_pred.extend(chunk_y_pred)

acc_score = accuracy_score(y_true, y_pred)

global_rec_score = recall_score(y_true, y_pred, average='micro')
class_rec_score = recall_score(y_true, y_pred, average='macro')

print(f'Test accuracy: {np.mean(acc_score)}')
print(f'Global test recall: {np.mean(global_rec_score)}')
print(f'Class test recall: {class_rec_score}')
